<!DOCTYPE html>
<html>
<body>
<h4>vaping graph tests</h4>

<h5>Multihost</h5>
<script src="/graph?hosts=all&nocache=1" type="text/javascript"></script>
<script src="/graph?hosts=all&nocache=2" type="text/javascript"></script>

<h5>Detail</h5>
<script src="/graph?hosts=204.42.254.199&type=detail" type="text/javascript"></script>

</body>
</html>
