<!DOCTYPE html>
<html>
<head>
  <title>vaping overview</title>
  <script src="{{sf}}/base/js/jquery.js" type="text/javascript"></script>
  <link rel="stylesheet" href="{{sf}}/base/css/bootstrap.css" type="text/css" />
  <link rel="stylesheet" href="/module_media/vaping.oview/1.0.0/main.css" type="text/css" />
</head>
<body>

<div class="vaping overview">
  <div class="container-fluid">
    
    <!-- ROW 1: START -->
    
    <div class="row height6">
      
      <div class="col-xs-12 col-md-4">
        <!-- COLUMN 1: OVERVIEW GRAPH 1 -->
        <script src="/graph?hosts=all&id=1&fit=yes" type="text/javascript"></script>
      </div>
      <div class="col-xs-12 col-md-4">
        <!-- COLUMN 2: OVERVIEW GRAPH 2 -->
        <script src="/graph?hosts=all&id=2&fit=yes" type="text/javascript"></script>
      </div>
       <div class="col-xs-12 col-md-4">
        <!-- COLUMN 3: OVERVIEW GRAPH 3 -->
        <script src="/graph?hosts=all&id=3&fit=yes" type="text/javascript"></script>
      </div>

    </div>

    <!-- ROW 1: END -->

    <!-- ROW 2: START -->
    
    <div class="row height2">
      
      <div class="col-xs-12 col-md-3">
        <!-- COLUMN 1: DETAIL GRAPH 1 -->
        <script src="/graph?hosts=204.42.254.199&id=2.1&fit=yes&type=detail" type="text/javascript"></script>
      </div>
      <div class="col-xs-12 col-md-3">
        <!-- COLUMN 2: DETAIL GRAPH 2 -->
        <script src="/graph?hosts=204.130.133.167&id=2.2&fit=yes&type=detail" type="text/javascript"></script>
      </div>
      <div class="col-xs-12 col-md-3">
        <!-- COLUMN 3: DETAIL GRAPH 3 -->
        <script src="/graph?hosts=46.18.174.22&id=2.3&fit=yes&type=detail" type="text/javascript"></script>
      </div>
      <div class="col-xs-12 col-md-3">
        <!-- COLUMN 4: DETAIL GRAPH 4 -->
        <script src="/graph?hosts=204.42.254.199&id=2.4&fit=yes&type=detail" type="text/javascript"></script>
      </div>
 
    </div>

    <!-- ROW 2: END -->

    <!-- ROW 2: START -->
    
    <div class="row height2">
 
      <div class="col-xs-12 col-md-3">
        <!-- COLUMN 3: DETAIL GRAPH 3 -->
        <script src="/graph?hosts=1.2.3.4&id=3.1&fit=yes&type=detail" type="text/javascript"></script>
      </div>
      <div class="col-xs-12 col-md-3">
        <!-- COLUMN 3: DETAIL GRAPH 3 -->
        <script src="/graph?hosts=1.2.3.4&id=3.2&fit=yes&type=detail" type="text/javascript"></script>
      </div>
      <div class="col-xs-12 col-md-3">
        <!-- COLUMN 3: DETAIL GRAPH 3 -->
        <script src="/graph?hosts=1.2.3.4&id=3.3&fit=yes&type=detail" type="text/javascript"></script>
      </div>
      <div class="col-xs-12 col-md-3">
        <!-- COLUMN 3: DETAIL GRAPH 3 -->
        <script src="/graph?hosts=1.2.3.4&id=3.4&fit=yes&type=detail" type="text/javascript"></script>
      </div>


    </div>

    <!-- ROW 2: END -->


  </div>
</div>

</body>
</html>
