<div class="row" style="height: {{ row.height }}%">
{% for col in row.cols %}
  <div class="col-xs-12 col-md-{{ col.width }} column" style="height:100%">
  {% if col.graph and col.graph.source %}
    {% if layout.type == "index" %}
    <div class="graph-label">
      {% if col.graph.config_dict.inspect %}
      <a href="/view/{{ col.graph.config_dict.inspect_layout }}/{{ col.graph.source }}">{{ col.graph.source }}</a>
      {% else %}
      <a>{{ col.graph.source }}</a>
      {% endif %}
    </div>
    {% endif %}
    <script src="/graph?targets={{ col.graph.targets }}&id={{ col.graph.id }}&fit={{ col.graph.fit }}&type={{ col.graph.config_dict.type }}{% if col.graph.config %}&config={{ col.graph.config }}{% endif %}&source={{ col.graph.source }}"></script>
  {% elif col.text %}
    <script type="text/javascript">
    graphsrvOverview.initTextMonitor('{{ col.text.name }}')
    </script>
  {% elif col.plugin %}
    <script type="text/javascript">
    graphsrvPlugins.init({{ col.plugin }});
    </script>
  {% else %}
    <div class="empty"></div>
  {% endif %}
  </div>
{% endfor %}
</div>
